"""
这个配置文件是自动生成到你的项目根目录的。
"""

import sys

# 项目根目录文件夹，这个一般不需要改，会根据PYTHONPATH智能获取。
# pycahrm自动添加了项目根目录到第一个PYTHONPATH，如果是cmd命令启动这先设置PYTHONPATH环境变量。
# windows设置  set PYTHONPATH=你当前python项目根目录,然后敲击你的python运行命令
# linux设置    export PYTHONPATH=你当前python项目根目录,然后敲击你的python运行命令
PYTHON_PROJ_DIR_LOCAL = sys.path[1]

# 这是远程机器的账号密码配置。把这个配置文件加到gitignore就不会泄漏了。
HOST = '192.168.6.133'
PORT = 22
USER = 'ydf'
PASSWORD = '123456'

PYTHON_INTERPRETER = 'python3'  # 如果你安装了四五个python环境，可以直接指定远程解释器的绝对路径  例如 /opt/minicondadir/ens/env35/python

FORBID_DEPLOY_FROM_LINUX = True # 一般生产机器是linux，是否禁止从linux部署到别的机器，这样可以防止你从生产环境远程到测试环境，配置后，即使生产环境的代码有远程部署，也不会执行远程部署而是直接运行。

# 上传文件夹的配置，具体可以看paramiko_util.py里面的代码。
PATH_PATTERN_EXLUDED_TUPLE = ('/.git/', '/.idea/', '/dist/', '/build/')  # 路径中如果有这些就自动过滤不上传
FILE_SUFFIX_TUPLE_EXLUDED = ('.pyc', '.log', '.gz')  # 这些后缀的文件不上传
ONLY_UPLOAD_WITHIN_THE_LAST_MODIFY_TIME = 3650 * 24 * 60 * 60  # 只有在这个时间之内修改的文件才上传。如果项目比较大，可以第一次完整上传，之后再把这个时间改小。
FILE_VOLUME_LIMIT = 1000 * 1000  # 大于这个体积的文件不上传，单位b。
SFTP_LOG_LEVEL = 20  # 文件夹上传时候的日志级别。10 logging.DEBUG ,20 logging.INFO 30 logging.WaRNING,如果要看为什么某个文件上传失败，可以设置debug级别。

EXTRA_SHELL_STR = ''  # 远程执行命令之前，可以自定义执行的shell语句，一般例如可以设置啥环境变量什么的。



# -*- coding: utf-8 -*-
# @Author  : ydf
# @Time    : 2020/4/11 0011 0:56
"""

使用覆盖的方式，做配置。
"""
import sys
import importlib
from pathlib import Path
from shutil import copyfile
import nb_log  # noqa
from auto_run_on_remote import remote_config

PACKAGE_NAME = 'auto_run_on_remote'
DEFAULT_CONFIG_MODLUE = 'remote_config'
CUSTOM_CONFIG_MODULE_NAME = 'auto_run_on_remote_config'

custom_config_module_path = Path(sys.path[1]) / Path(f'{CUSTOM_CONFIG_MODULE_NAME}.py')


def use_config_form_config_module():
    """
    自动读取配置。会优先读取启动脚本的目录的distributed_frame_config.py文件。没有则读取项目根目录下的distributed_frame_config.py
    :return:
    """

    try:
        m = importlib.import_module(CUSTOM_CONFIG_MODULE_NAME)
        msg = f'{PACKAGE_NAME} 包 读取到\n "{m.__file__}:1" 文件里面的变量作为优先配置了\n'
        print(msg)
        for var_namex, var_valuex in m.__dict__.items():
            if var_namex.isupper():
                setattr(remote_config, var_namex, var_valuex)
    except ModuleNotFoundError:
        auto_creat_config_file_to_project_root_path()
        msg = f'''在你的项目根目录下生成了 \n "{custom_config_module_path}:1" 的auto_run_on_remote包的远程上传项目代码的配置文件，快去看看并修改一些自定义配置吧'''
        print(msg)


def auto_creat_config_file_to_project_root_path():
    # print(Path(sys.path[1]).as_posix())
    # print((Path(__file__).parent.parent).absolute().as_posix())
    """
    :return:
    """
    # if Path(sys.path[1]).as_posix() in Path(__file__).parent.absolute().as_posix():
    #     print('不希望在本项目里面创建')
    #     return
    # noinspection PyPep8
    """
        如果没设置PYTHONPATH，sys.path会这样，取第一个就会报错
        ['', '/data/miniconda3dir/inner/envs/mtfy/lib/python36.zip', '/data/miniconda3dir/inner/envs/mtfy/lib/python3.6', '/data/miniconda3dir/inner/envs/mtfy/lib/python3.6/lib-dynload', '/root/.local/lib/python3.6/site-packages', '/data/miniconda3dir/inner/envs/mtfy/lib/python3.6/site-packages']
        
        ['', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\python36.zip', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\DLLs', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib', 'F:\\minicondadir\\Miniconda2\\envs\\py36', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\multiprocessing_log_manager-0.2.0-py3.6.egg', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\pyinstaller-3.4-py3.6.egg', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\pywin32_ctypes-0.2.0-py3.6.egg', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\altgraph-0.16.1-py3.6.egg', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\macholib-1.11-py3.6.egg', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\pefile-2019.4.18-py3.6.egg', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\win32', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\win32\\lib', 'F:\\minicondadir\\Miniconda2\\envs\\py36\\lib\\site-packages\\Pythonwin']
    """
    if '/lib/python' in sys.path[1] or r'\lib\python' in sys.path[1] or '.zip' in sys.path[1]:
        raise EnvironmentError('''如果用pycahrm启动，默认不需要你手动亲自设置PYTHONPATH，如果你是cmd或者shell中直接敲击python xx.py 来运行，
                               报现在这个错误，你现在肯定是没有设置PYTHONPATH环境变量，不要设置永久环境变量，设置临时会话环境变量就行，
                               windows设置  set PYTHONPATH=你当前python项目根目录,然后敲击你的python运行命令    
                               linux设置    export PYTHONPATH=你当前python项目根目录,然后敲击你的python运行命令    
                               要是连PYTHONPATH这个知识点都不知道，那就要google 百度去学习PYTHONPATH作用了，非常重要非常好用，
                               不知道PYTHONPATH作用的人，在深层级文件夹作为运行起点导入外层目录的包的时候，如果把深层级文件作为python的执行文件起点，经常需要到处很low的手写 sys.path.insert硬编码，这种方式写代码太low了。
                               知道PYTHONPATH的人无论项目有多少层级的文件夹，无论是多深层级文件夹导入外层文件夹，代码里面永久都不需要出现手动硬编码操纵sys.path.append
                               ''')
    # with (Path(sys.path[1]) / Path('nb_log_config.py')).open(mode='w', encoding='utf8') as f:
    #     f.write(config_file_content)
    copyfile(Path(__file__).parent / Path(f'{DEFAULT_CONFIG_MODLUE}.py'), Path(sys.path[1]) / Path(f'{CUSTOM_CONFIG_MODULE_NAME}.py'))


use_config_form_config_module()



# coding=utf-8

from setuptools import setup, find_packages

# with open("README.md", "r",encoding='utf8') as fh:
#     long_description = fh.read()

setup(
    name='auto_run_on_remote',  #
    version=0.2,
    description=(
        'auto run current python script on remote server'
    ),
    # long_description=open('README.md', 'r',encoding='utf8').read(),
    keywords=("auto_run_on_remote"),
    long_description_content_type="text/markdown",
    long_description=open('README.md', 'r', encoding='utf8').read(),
    author='bfzs',
    author_email='ydf0509@sohu.com',
    maintainer='ydf',
    maintainer_email='ydf0509@sohu.com',
    license='BSD License',
    # packages=['douban'], #
    packages=find_packages(),
    # packages=['function_scheduling_distributed_framework'], # 这样内层级文件夹的没有打包进去。
    include_package_data=True,
    platforms=["all"],
    url='https://github.com/ydf0509/distributed_framework',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'nb_log'
        'fabric2==2.6.0'  #有的机器包rust错误， 这样做 curl https://sh.rustup.rs -sSf | sh
    ]
)
"""
官方 https://pypi.org/simple
清华 https://pypi.tuna.tsinghua.edu.cn/simple
豆瓣 https://pypi.douban.com/simple/ 
阿里云 https://mirrors.aliyun.com/pypi/simple/

打包上传
python setup.py sdist upload -r pypi

# python setup.py bdist_wheel
python setup.py bdist_wheel & twine upload dist/auto_run_on_remote-11.7-py3-none-any.whl
python setup.py sdist & twine upload dist/auto_run_on_remote-0.2.tar.gz

最快的下载方式，上传立即可安装。阿里云源同步官网pypi间隔要等很久。
./pip install auto_run_on_remote==3.5 -i https://pypi.org/simple   
最新版下载
./pip install auto_run_on_remote --upgrade -i https://pypi.org/simple       
"""




import sys
import re
import os
import time
import nb_log
from auto_run_on_remote.paramiko_util import ParamikoFolderUploader
from fabric2 import Connection
from auto_run_on_remote import remote_config
from auto_run_on_remote import set_config  # noqa

logger = nb_log.get_logger('run_script_on_remote_server')
python_proj_dir = remote_config.PYTHON_PROJ_DIR_LOCAL.replace('\\', '/')
if not python_proj_dir.endswith('/'):
    python_proj_dir += '/'
python_proj_dir_short = python_proj_dir.split('/')[-2]

if remote_config.USER == 'root':  # 文件夹会被自动创建，无需用户创建。
    remote_dir = f'/pycodes/{python_proj_dir_short}/'
else:
    remote_dir = f'/home/{remote_config.USER}/pycodes/{python_proj_dir_short}/'


def run_current_script_on_remote():
    if remote_config.FORBID_DEPLOY_FROM_LINUX and os.name == 'posix':
        # 一般生产机器是linux，是否禁止从linux部署到别的机器，这样可以防止你从生产环境远程到测试环境，配置后，即使生产环境的代码有远程部署，也不会执行远程部署而是直接运行。
        return
    if int(os.getenv('is_auto_remote_run', 0)) == 1:  # 不能循环递归远程启动。
        return
    logger.warning(f'将本地文件夹代码 {python_proj_dir}  上传到远程 {remote_config.HOST} 的 {remote_dir} 文件夹。')
    t_start = time.perf_counter()
    uploader = ParamikoFolderUploader(remote_config.HOST, remote_config.PORT, remote_config.USER, remote_config.PASSWORD,
                                      python_proj_dir, remote_dir,
                                      path_pattern_exluded_tuple=remote_config.PATH_PATTERN_EXLUDED_TUPLE,
                                      file_suffix_tuple_exluded=remote_config.FILE_SUFFIX_TUPLE_EXLUDED,
                                      only_upload_within_the_last_modify_time=remote_config.ONLY_UPLOAD_WITHIN_THE_LAST_MODIFY_TIME,
                                      file_volume_limit=remote_config.FILE_VOLUME_LIMIT, sftp_log_level=remote_config.SFTP_LOG_LEVEL)

    uploader.upload()
    logger.info(f'上传 本地文件夹代码 {python_proj_dir}  上传到远程 {remote_config.HOST} 的 {remote_dir} 文件夹耗时 {round(time.perf_counter() - t_start, 3)} 秒')
    # conn.run(f'''export PYTHONPATH={remote_dir}:$PYTHONPATH''')
    # 获取被调用函数所在模块文件名
    # print(sys._getframe())
    local_file_name = sys._getframe(1).f_code.co_filename.replace('\\', '/')  # noqa
    # file_name = re.sub(f'^{python_proj_dir}', '', local_file_name)
    file_name = re.sub(f'^{python_proj_dir}', remote_dir, local_file_name)  # 远程文件名字。
    process_mark = f'auto_remote_run_mark__{file_name.replace("/", "__")[:-3]}'

    conn = Connection(remote_config.HOST, port=remote_config.PORT, user=remote_config.USER, connect_kwargs={"password": remote_config.PASSWORD}, )
    kill_shell = f'''ps -aux|grep {process_mark}|grep -v grep|awk '{{print $2}}' |xargs kill -9'''
    logger.warning(f'{kill_shell} 命令杀死 {process_mark} 标识的进程')
    uploader.ssh.exec_command(kill_shell)
    # conn.run(kill_shell, encoding='utf-8')

    python_exec_str = f''' {remote_config.PYTHON_INTERPRETER} {file_name}  -auto_remote_process_mark {process_mark} '''
    shell_str = f'''export is_auto_remote_run=1;export PYTHONPATH={remote_dir}:$PYTHONPATH ;cd {remote_dir}; {python_exec_str}'''
    extra_shell_str2 = remote_config.EXTRA_SHELL_STR  # 内部函数对外部变量不能直接改。
    if not extra_shell_str2.endswith(';') and remote_config.EXTRA_SHELL_STR != '':
        extra_shell_str2 += ';'
    shell_str = extra_shell_str2 + shell_str
    logger.warning(f'使用语句 {shell_str} 在远程机器 {remote_config.HOST} 上启动脚本 {file_name}')
    conn.run(shell_str, encoding='utf-8')
    sys.exit()  # 使本机不执行代码。
